@@ -1,23 +1,25 @@
-#include <cstdio>
 /* === S Y N F I G ========================================================= */
 /*!	\file activepoint.h
 **	\brief Template Header
 **
-**	$Id$
-**
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **	Copyright (c) 2007 Chris Moore
 **
-**	This package is free software; you can redistribute it and/or
-**	modify it under the terms of the GNU General Public License as
-**	published by the Free Software Foundation; either version 2 of
-**	the License, or (at your option) any later version.
+**	This file is part of Synfig.
 **
-**	This package is distributed in the hope that it will be useful,
+**	Synfig is free software: you can redistribute it and/or modify
+**	it under the terms of the GNU General Public License as published by
+**	the Free Software Foundation, either version 2 of the License, or
+**	(at your option) any later version.
+**
+**	Synfig is distributed in the hope that it will be useful,
 **	but WITHOUT ANY WARRANTY; without even the implied warranty of
-**	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-**	General Public License for more details.
+**	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+**	GNU General Public License for more details.
+**
+**	You should have received a copy of the GNU General Public License
+**	along with Synfig.  If not, see <https://www.gnu.org/licenses/>.
 **	\endlegal
 */
 /* ========================================================================= */
@@ -29,10 +31,11 @@
 
 /* === H E A D E R S ======================================================= */
 
-#include "time.h"
-#include "uniqueid.h"
 #include <ETL/handle>
 
+#include <synfig/time.h>
+#include <synfig/uniqueid.h>
+
 /* === M A C R O S ========================================================= */
 
 /* === T Y P E D E F S ===================================================== */
@@ -59,8 +62,8 @@ public:
 	//! Does this activepoint turn the entry on, or off?
 	bool state;
 
-	bool operator<(const Activepoint& rhs) { return time<rhs.time; }
-	bool operator<(const Time& rhs) { return time<rhs; }
+	bool operator<(const Activepoint& rhs) const { return time<rhs.time; }
+	bool operator<(const Time& rhs) const { return time<rhs; }
 
 	Activepoint(const Time &time, const bool &state, int p=0):
 		index(), time(time), priority(p),state(state) { }
