@@ -2,20 +2,23 @@
 /*!	\file filesystemnative.h
 **	\brief FileSystemNative
 **
-**	$Id$
-**
 **	\legal
 **	......... ... 2013 Ivan Mahonin
 **
-**	This package is free software; you can redistribute it and/or
-**	modify it under the terms of the GNU General Public License as
-**	published by the Free Software Foundation; either version 2 of
-**	the License, or (at your option) any later version.
+**	This file is part of Synfig.
 **
-**	This package is distributed in the hope that it will be useful,
+**	Synfig is free software: you can redistribute it and/or modify
+**	it under the terms of the GNU General Public License as published by
+**	the Free Software Foundation, either version 2 of the License, or
+**	(at your option) any later version.
+**
+**	Synfig is distributed in the hope that it will be useful,
 **	but WITHOUT ANY WARRANTY; without even the implied warranty of
-**	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-**	General Public License for more details.
+**	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+**	GNU General Public License for more details.
+**
+**	You should have received a copy of the GNU General Public License
+**	along with Synfig.  If not, see <https://www.gnu.org/licenses/>.
 **	\endlegal
 */
 /* ========================================================================= */
@@ -51,7 +54,8 @@ namespace synfig
 			friend class FileSystemNative;
 			FILE *file_;
 			ReadStream(FileSystem::Handle file_system, FILE *file);
-			virtual size_t internal_read(void *buffer, size_t size);
+			size_t internal_read(void *buffer, size_t size) override;
+			int pbackfail(int ch) override;
 			std::istream::pos_type seekpos(std::istream::pos_type pos, std::ios_base::openmode which = std::ios_base::in | std::ios_base::out) override;
 		public:
 			virtual ~ReadStream();
@@ -65,7 +69,7 @@ namespace synfig
 			friend class FileSystemNative;
 			FILE *file_;
 			WriteStream(FileSystem::Handle file_system, FILE *file_);
-			virtual size_t internal_write(const void *buffer, size_t size);
+			size_t internal_write(const void *buffer, size_t size) override;
 		public:
 			virtual ~WriteStream();
 		};
